@@ -1,6 +1,6 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
- * Copyright (c) 1991-1998 University of Maryland at College Park
+ * Copyright (c) 1991-1999 University of Maryland at College Park
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -25,7 +25,7 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id: dgram.h,v 1.6.2.2.4.1.2.1 2002/03/24 19:23:23 jrjackson Exp $
+ * $Id: dgram.h,v 1.13 2001/07/13 22:38:05 jrjackson Exp $
  *
  * interface for datagram module
  */
@@ -54,11 +54,8 @@ int dgram_send P((char *hostname, int port, dgram_t *dgram));
 int dgram_send_addr P((struct sockaddr_in addr, dgram_t *dgram));
 int dgram_recv P((dgram_t *dgram, int timeout, struct sockaddr_in *fromaddr));
 void dgram_zero P((dgram_t *dgram));
-void dgram_cat P((dgram_t *dgram, const char *str));
+void dgram_cat P((dgram_t *dgram, const char *fmt, ...))
+    __attribute__ ((format (printf, 2, 3)));
 void dgram_eatline P((dgram_t *dgram));
 
-extern dgram_t  *debug_dgram_alloc  P((char *c, int l));
-
-#define	dgram_alloc()		debug_dgram_alloc(__FILE__, __LINE__)
-
 #endif /* ! DGRAM_H */
