@@ -25,11 +25,13 @@
  *			   University of Maryland at College Park
  */
 /*
- * $Id: driverio.h,v 1.13.2.7.4.3 2001/11/08 18:44:56 martinea Exp $
+ * $Id: driverio.h,v 1.32 2005/12/03 13:27:43 martinea Exp $
  *
  * driver-related helper functions
  */
 
+#include "event.h"
+
 #include "holding.h"
 #include "server_util.h"
 
@@ -39,14 +41,30 @@
 #define GLOBAL extern
 #endif
 
+/* chunker process structure */
+
+typedef struct chunker_s {
+    char *name;			/* name of this chunker */
+    int pid;			/* its pid */
+    int down;			/* state */
+    int fd;			/* read/write */
+    int result;
+    event_handle_t *ev_read;	/* read event handle */
+    struct dumper_s *dumper;
+} chunker_t;
+
 /* dumper process structure */
 
 typedef struct dumper_s {
-    char *name;		/* name of this dumper */
-    int pid;		/* its pid */
-    int busy, down;
-    int infd, outfd;
-    disk_t *dp;
+    char *name;			/* name of this dumper */
+    int pid;			/* its pid */
+    int busy, down;		/* state */
+    int fd;			/* read/write */
+    int result;
+    int output_port;		/* output port */
+    event_handle_t *ev_read;	/* read event handle */
+    disk_t *dp;			/* disk currently being dumped */
+    chunker_t *chunker;
 } dumper_t;
 
 typedef struct assignedhd_s {
@@ -63,6 +81,8 @@ typedef struct sched_s {
     int level, degr_level;
     long est_time, degr_time;
     unsigned long est_size, degr_size, act_size;
+    unsigned long origsize, dumpsize;
+    unsigned long dumptime, tapetime;
     char *dumpdate, *degr_dumpdate;
     int est_kps, degr_kps;
     char *destname;				/* file/port name */
@@ -87,20 +107,27 @@ typedef struct holdalloc_s {
 #define holdalloc(hp)	((holdalloc_t *) (hp)->up)
 
 GLOBAL dumper_t dmptable[MAX_DUMPERS];
+GLOBAL chunker_t chktable[MAX_DUMPERS];
+
+/* command/result tokens */
 
-GLOBAL int maxfd;
-GLOBAL fd_set readset;
 GLOBAL int taper, taper_busy, taper_pid;
+GLOBAL event_handle_t *taper_ev_read;
 
-void init_driverio();
+void init_driverio P((void));
 void startup_tape_process P((char *taper_program));
 void startup_dump_process P((dumper_t *dumper, char *dumper_program));
 void startup_dump_processes P((char *dumper_program, int inparallel));
+void startup_chunk_process P((chunker_t *chunker, char *chunker_program));
+
 cmd_t getresult P((int fd, int show, int *result_argc, char **result_argv, int max_arg));
 int taper_cmd P((cmd_t cmd, void *ptr, char *destname, int level, char *datestamp));
 int dumper_cmd P((dumper_t *dumper, cmd_t cmd, disk_t *dp));
+int chunker_cmd P((chunker_t *chunker, cmd_t cmd, disk_t *dp));
 disk_t *serial2disk P((char *str));
 void free_serial P((char *str));
+void free_serial_dp P((disk_t *dp));
+void check_unfree_serial P(());
 char *disk2serial P((disk_t *dp));
 void update_info_dumper P((disk_t *dp, long origsize, long dumpsize, long dumptime));
 void update_info_taper P((disk_t *dp, char *label, int filenum, int level));
